#
# This is an auto-generated file.  DO NOT EDIT!
#
# pylint: disable=line-too-long

from ansys.fluent.core.services.datamodel_se import (
    PyMenu,
    PyParameter,
    PyTextual,
    PyNumerical,
    PyDictionary,
    PyNamedObjectContainer,
    PyCommand,
    PyQuery,
    PyCommandArguments,
    PyTextualCommandArgumentsSubItem,
    PyNumericalCommandArgumentsSubItem,
    PyDictionaryCommandArgumentsSubItem,
    PyParameterCommandArgumentsSubItem,
    PySingletonCommandArgumentsSubItem
)


class Root(PyMenu):
    """
    Singleton Root.
    """
    def __init__(self, service, rules, path):
        self.CellZone = self.__class__.CellZone(service, rules, path + [("CellZone", "")])
        self.FaceZone = self.__class__.FaceZone(service, rules, path + [("FaceZone", "")])
        self.Zone = self.__class__.Zone(service, rules, path + [("Zone", "")])
        self.GlobalSettings = self.__class__.GlobalSettings(service, rules, path + [("GlobalSettings", "")])
        self.ZoneList = self.__class__.ZoneList(service, rules, path + [("ZoneList", "")])
        self.JournalCommand = self.__class__.JournalCommand(service, rules, "JournalCommand", path)
        self.TWF_AssociateMesh = self.__class__.TWF_AssociateMesh(service, rules, "TWF_AssociateMesh", path)
        self.TWF_BasicMachineDescription = self.__class__.TWF_BasicMachineDescription(service, rules, "TWF_BasicMachineDescription", path)
        self.TWF_BladeRowAnalysisScope = self.__class__.TWF_BladeRowAnalysisScope(service, rules, "TWF_BladeRowAnalysisScope", path)
        self.TWF_CompleteWorkflowSetup = self.__class__.TWF_CompleteWorkflowSetup(service, rules, "TWF_CompleteWorkflowSetup", path)
        self.TWF_CreateCFDModel = self.__class__.TWF_CreateCFDModel(service, rules, "TWF_CreateCFDModel", path)
        self.TWF_ImportMesh = self.__class__.TWF_ImportMesh(service, rules, "TWF_ImportMesh", path)
        self.TWF_MapRegionInfo = self.__class__.TWF_MapRegionInfo(service, rules, "TWF_MapRegionInfo", path)
        self.TWF_ReportDefMonitors = self.__class__.TWF_ReportDefMonitors(service, rules, "TWF_ReportDefMonitors", path)
        self.TWF_TurboPhysics = self.__class__.TWF_TurboPhysics(service, rules, "TWF_TurboPhysics", path)
        self.TWF_TurboRegionsZones = self.__class__.TWF_TurboRegionsZones(service, rules, "TWF_TurboRegionsZones", path)
        self.TWF_TurboSurfaces = self.__class__.TWF_TurboSurfaces(service, rules, "TWF_TurboSurfaces", path)
        self.TWF_TurboTopology = self.__class__.TWF_TurboTopology(service, rules, "TWF_TurboTopology", path)
        super().__init__(service, rules, path)

    class CellZone(PyNamedObjectContainer):
        """
        .
        """
        class _CellZone(PyMenu):
            """
            Singleton _CellZone.
            """
            def __init__(self, service, rules, path):
                self.ChildZones = self.__class__.ChildZones(service, rules, path + [("ChildZones", "")])
                self.ConnectedFaces = self.__class__.ConnectedFaces(service, rules, path + [("ConnectedFaces", "")])
                self.NameInMesh = self.__class__.NameInMesh(service, rules, path + [("NameInMesh", "")])
                self.ParentZone = self.__class__.ParentZone(service, rules, path + [("ParentZone", "")])
                self.UnambiguousName = self.__class__.UnambiguousName(service, rules, path + [("UnambiguousName", "")])
                self._name_ = self.__class__._name_(service, rules, path + [("_name_", "")])
                super().__init__(service, rules, path)

            class ChildZones(PyTextual):
                """
                Parameter ChildZones of value type list[str].
                """
                pass

            class ConnectedFaces(PyTextual):
                """
                Parameter ConnectedFaces of value type list[str].
                """
                pass

            class NameInMesh(PyTextual):
                """
                Parameter NameInMesh of value type str.
                """
                pass

            class ParentZone(PyTextual):
                """
                Parameter ParentZone of value type str.
                """
                pass

            class UnambiguousName(PyTextual):
                """
                Parameter UnambiguousName of value type str.
                """
                pass

            class _name_(PyTextual):
                """
                Parameter _name_ of value type str.
                """
                pass

        def __getitem__(self, key: str) -> _CellZone:
            return super().__getitem__(key)

    class FaceZone(PyNamedObjectContainer):
        """
        .
        """
        class _FaceZone(PyMenu):
            """
            Singleton _FaceZone.
            """
            def __init__(self, service, rules, path):
                self.ChildZones = self.__class__.ChildZones(service, rules, path + [("ChildZones", "")])
                self.NameInMesh = self.__class__.NameInMesh(service, rules, path + [("NameInMesh", "")])
                self.ParentZone = self.__class__.ParentZone(service, rules, path + [("ParentZone", "")])
                self.UnambiguousName = self.__class__.UnambiguousName(service, rules, path + [("UnambiguousName", "")])
                self._name_ = self.__class__._name_(service, rules, path + [("_name_", "")])
                super().__init__(service, rules, path)

            class ChildZones(PyTextual):
                """
                Parameter ChildZones of value type list[str].
                """
                pass

            class NameInMesh(PyTextual):
                """
                Parameter NameInMesh of value type str.
                """
                pass

            class ParentZone(PyTextual):
                """
                Parameter ParentZone of value type str.
                """
                pass

            class UnambiguousName(PyTextual):
                """
                Parameter UnambiguousName of value type str.
                """
                pass

            class _name_(PyTextual):
                """
                Parameter _name_ of value type str.
                """
                pass

        def __getitem__(self, key: str) -> _FaceZone:
            return super().__getitem__(key)

    class Zone(PyNamedObjectContainer):
        """
        .
        """
        class _Zone(PyMenu):
            """
            Singleton _Zone.
            """
            def __init__(self, service, rules, path):
                self.ChildZones = self.__class__.ChildZones(service, rules, path + [("ChildZones", "")])
                self.NameInMesh = self.__class__.NameInMesh(service, rules, path + [("NameInMesh", "")])
                self.ParentZone = self.__class__.ParentZone(service, rules, path + [("ParentZone", "")])
                self.UnambiguousName = self.__class__.UnambiguousName(service, rules, path + [("UnambiguousName", "")])
                self._name_ = self.__class__._name_(service, rules, path + [("_name_", "")])
                super().__init__(service, rules, path)

            class ChildZones(PyTextual):
                """
                Parameter ChildZones of value type list[str].
                """
                pass

            class NameInMesh(PyTextual):
                """
                Parameter NameInMesh of value type str.
                """
                pass

            class ParentZone(PyTextual):
                """
                Parameter ParentZone of value type str.
                """
                pass

            class UnambiguousName(PyTextual):
                """
                Parameter UnambiguousName of value type str.
                """
                pass

            class _name_(PyTextual):
                """
                Parameter _name_ of value type str.
                """
                pass

        def __getitem__(self, key: str) -> _Zone:
            return super().__getitem__(key)

    class GlobalSettings(PyMenu):
        """
        Singleton GlobalSettings.
        """
        def __init__(self, service, rules, path):
            self.EnableTurboMeshing = self.__class__.EnableTurboMeshing(service, rules, path + [("EnableTurboMeshing", "")])
            super().__init__(service, rules, path)

        class EnableTurboMeshing(PyParameter):
            """
            Parameter EnableTurboMeshing of value type bool.
            """
            pass

    class ZoneList(PyMenu):
        """
        Singleton ZoneList.
        """
        def __init__(self, service, rules, path):
            self.CellZones = self.__class__.CellZones(service, rules, path + [("CellZones", "")])
            self.FaceZones = self.__class__.FaceZones(service, rules, path + [("FaceZones", "")])
            super().__init__(service, rules, path)

        class CellZones(PyTextual):
            """
            Parameter CellZones of value type list[str].
            """
            pass

        class FaceZones(PyTextual):
            """
            Parameter FaceZones of value type list[str].
            """
            pass

    class JournalCommand(PyCommand):
        """
        Command JournalCommand.

        Parameters
        ----------
        JournalString : str
        PythonJournal : bool

        Returns
        -------
        bool
        """
        class _JournalCommandCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.JournalString = self._JournalString(self, "JournalString", service, rules, path)
                self.PythonJournal = self._PythonJournal(self, "PythonJournal", service, rules, path)

            class _JournalString(PyTextualCommandArgumentsSubItem):
                """
                Argument JournalString.
                """

            class _PythonJournal(PyParameterCommandArgumentsSubItem):
                """
                Argument PythonJournal.
                """

        def create_instance(self) -> _JournalCommandCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._JournalCommandCommandArguments(*args)

    class TWF_AssociateMesh(PyCommand):
        """
        Command TWF_AssociateMesh.

        Parameters
        ----------
        AMChildName : str
        AMSelectComponentScope : str
        UseWireframe : bool
            Toggle the display of the model in wireframe.
        RenameCellZones : str
            Determines how your zones names appear once this task is complete, depending on your preferences. When set to Yes, using row names, this field will change the associated cell (or face) zone name according to the corresponding Name. When set to Yes, using row numbers, this field will change the associated cell (or face) zone name according to the corresponding Row number. You can also choose No to keep the zone names as they are.
        DefaultAMRowNumList : list[str]
        DefaultAMCellZonesList : list[str]
        AMRowNumList : list[str]
        OldAMCellZonesList : list[str]
        NewAMCellZonesList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_AssociateMeshCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.AMChildName = self._AMChildName(self, "AMChildName", service, rules, path)
                self.AMSelectComponentScope = self._AMSelectComponentScope(self, "AMSelectComponentScope", service, rules, path)
                self.UseWireframe = self._UseWireframe(self, "UseWireframe", service, rules, path)
                self.RenameCellZones = self._RenameCellZones(self, "RenameCellZones", service, rules, path)
                self.DefaultAMRowNumList = self._DefaultAMRowNumList(self, "DefaultAMRowNumList", service, rules, path)
                self.DefaultAMCellZonesList = self._DefaultAMCellZonesList(self, "DefaultAMCellZonesList", service, rules, path)
                self.AMRowNumList = self._AMRowNumList(self, "AMRowNumList", service, rules, path)
                self.OldAMCellZonesList = self._OldAMCellZonesList(self, "OldAMCellZonesList", service, rules, path)
                self.NewAMCellZonesList = self._NewAMCellZonesList(self, "NewAMCellZonesList", service, rules, path)

            class _AMChildName(PyTextualCommandArgumentsSubItem):
                """
                Argument AMChildName.
                """

            class _AMSelectComponentScope(PyTextualCommandArgumentsSubItem):
                """
                Argument AMSelectComponentScope.
                """

            class _UseWireframe(PyParameterCommandArgumentsSubItem):
                """
                Toggle the display of the model in wireframe.
                """

            class _RenameCellZones(PyTextualCommandArgumentsSubItem):
                """
                Determines how your zones names appear once this task is complete, depending on your preferences. When set to Yes, using row names, this field will change the associated cell (or face) zone name according to the corresponding Name. When set to Yes, using row numbers, this field will change the associated cell (or face) zone name according to the corresponding Row number. You can also choose No to keep the zone names as they are.
                """

            class _DefaultAMRowNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument DefaultAMRowNumList.
                """

            class _DefaultAMCellZonesList(PyTextualCommandArgumentsSubItem):
                """
                Argument DefaultAMCellZonesList.
                """

            class _AMRowNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument AMRowNumList.
                """

            class _OldAMCellZonesList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldAMCellZonesList.
                """

            class _NewAMCellZonesList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewAMCellZonesList.
                """

        def create_instance(self) -> _TWF_AssociateMeshCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_AssociateMeshCommandArguments(*args)

    class TWF_BasicMachineDescription(PyCommand):
        """
        Command TWF_BasicMachineDescription.

        Parameters
        ----------
        ComponentType : str
            Specify the type of machine component: either an Axial Turbine, Axial Compressor, a Radial Turbine, or a Radial Compressor.
        ComponentName : str
            Specify a name for the component, or use the default value.
        NumRows : int
            Specify the number of rows for the component. For each row, use the table to provide a Name, a Type (stationary or rotating), the Number of Blades, and whether or not there is a tip gap present (spacing between the blade and the hub/shroud).
        RowNumList : list[str]
        OldRowNameList : list[str]
        NewRowNameList : list[str]
        OldRowTypeList : list[str]
        NewRowTypeList : list[str]
        OldNumOfBladesList : list[str]
        NewNumOfBladesList : list[str]
        OldEnableTipGapList : list[str]
        NewEnableTipGapList : list[str]
        CombustorType : str

        Returns
        -------
        bool
        """
        class _TWF_BasicMachineDescriptionCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.ComponentType = self._ComponentType(self, "ComponentType", service, rules, path)
                self.ComponentName = self._ComponentName(self, "ComponentName", service, rules, path)
                self.NumRows = self._NumRows(self, "NumRows", service, rules, path)
                self.RowNumList = self._RowNumList(self, "RowNumList", service, rules, path)
                self.OldRowNameList = self._OldRowNameList(self, "OldRowNameList", service, rules, path)
                self.NewRowNameList = self._NewRowNameList(self, "NewRowNameList", service, rules, path)
                self.OldRowTypeList = self._OldRowTypeList(self, "OldRowTypeList", service, rules, path)
                self.NewRowTypeList = self._NewRowTypeList(self, "NewRowTypeList", service, rules, path)
                self.OldNumOfBladesList = self._OldNumOfBladesList(self, "OldNumOfBladesList", service, rules, path)
                self.NewNumOfBladesList = self._NewNumOfBladesList(self, "NewNumOfBladesList", service, rules, path)
                self.OldEnableTipGapList = self._OldEnableTipGapList(self, "OldEnableTipGapList", service, rules, path)
                self.NewEnableTipGapList = self._NewEnableTipGapList(self, "NewEnableTipGapList", service, rules, path)
                self.CombustorType = self._CombustorType(self, "CombustorType", service, rules, path)

            class _ComponentType(PyTextualCommandArgumentsSubItem):
                """
                Specify the type of machine component: either an Axial Turbine, Axial Compressor, a Radial Turbine, or a Radial Compressor.
                """

            class _ComponentName(PyTextualCommandArgumentsSubItem):
                """
                Specify a name for the component, or use the default value.
                """

            class _NumRows(PyNumericalCommandArgumentsSubItem):
                """
                Specify the number of rows for the component. For each row, use the table to provide a Name, a Type (stationary or rotating), the Number of Blades, and whether or not there is a tip gap present (spacing between the blade and the hub/shroud).
                """

            class _RowNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument RowNumList.
                """

            class _OldRowNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldRowNameList.
                """

            class _NewRowNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewRowNameList.
                """

            class _OldRowTypeList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldRowTypeList.
                """

            class _NewRowTypeList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewRowTypeList.
                """

            class _OldNumOfBladesList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldNumOfBladesList.
                """

            class _NewNumOfBladesList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewNumOfBladesList.
                """

            class _OldEnableTipGapList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldEnableTipGapList.
                """

            class _NewEnableTipGapList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewEnableTipGapList.
                """

            class _CombustorType(PyTextualCommandArgumentsSubItem):
                """
                Argument CombustorType.
                """

        def create_instance(self) -> _TWF_BasicMachineDescriptionCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_BasicMachineDescriptionCommandArguments(*args)

    class TWF_BladeRowAnalysisScope(PyCommand):
        """
        Command TWF_BladeRowAnalysisScope.

        Parameters
        ----------
        ASChildName : str
        ASSelectComponent : str
        ASRowNumList : list[str]
        OldASIncludeRowList : list[str]
        NewASIncludeRowList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_BladeRowAnalysisScopeCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.ASChildName = self._ASChildName(self, "ASChildName", service, rules, path)
                self.ASSelectComponent = self._ASSelectComponent(self, "ASSelectComponent", service, rules, path)
                self.ASRowNumList = self._ASRowNumList(self, "ASRowNumList", service, rules, path)
                self.OldASIncludeRowList = self._OldASIncludeRowList(self, "OldASIncludeRowList", service, rules, path)
                self.NewASIncludeRowList = self._NewASIncludeRowList(self, "NewASIncludeRowList", service, rules, path)

            class _ASChildName(PyTextualCommandArgumentsSubItem):
                """
                Argument ASChildName.
                """

            class _ASSelectComponent(PyTextualCommandArgumentsSubItem):
                """
                Argument ASSelectComponent.
                """

            class _ASRowNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument ASRowNumList.
                """

            class _OldASIncludeRowList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldASIncludeRowList.
                """

            class _NewASIncludeRowList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewASIncludeRowList.
                """

        def create_instance(self) -> _TWF_BladeRowAnalysisScopeCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_BladeRowAnalysisScopeCommandArguments(*args)

    class TWF_CompleteWorkflowSetup(PyCommand):
        """
        Command TWF_CompleteWorkflowSetup.


        Returns
        -------
        bool
        """
        class _TWF_CompleteWorkflowSetupCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)

        def create_instance(self) -> _TWF_CompleteWorkflowSetupCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_CompleteWorkflowSetupCommandArguments(*args)

    class TWF_CreateCFDModel(PyCommand):
        """
        Command TWF_CreateCFDModel.

        Parameters
        ----------
        CFDMChildName : str
        CFDMSelectMeshAssociation : str
        AxisOfRotation : str
            Specify the rotational axis for the generated CFD turbomachine geometry.
        DelayCFDModelCreation : bool
        RestrictToFactors : bool
            Choose whether or not to restrict the number of model blade sectors to a factor of the number of blades.
        EstimateNumBlades : bool
        CFDMRowNumList : list[str]
        OldCFDMNumOfBladesList : list[str]
        NewCFDMNumOfBladesList : list[str]
        OldCFDMModelBladesList : list[str]
        NewCFDMModelBladesList : list[str]
        OldCFDMAngleOffset : list[str]
        NewCFDMAngleOffset : list[str]
        OldCFDMBladesPerSectorList : list[str]
        NewCFDMBladesPerSectorList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_CreateCFDModelCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.CFDMChildName = self._CFDMChildName(self, "CFDMChildName", service, rules, path)
                self.CFDMSelectMeshAssociation = self._CFDMSelectMeshAssociation(self, "CFDMSelectMeshAssociation", service, rules, path)
                self.AxisOfRotation = self._AxisOfRotation(self, "AxisOfRotation", service, rules, path)
                self.DelayCFDModelCreation = self._DelayCFDModelCreation(self, "DelayCFDModelCreation", service, rules, path)
                self.RestrictToFactors = self._RestrictToFactors(self, "RestrictToFactors", service, rules, path)
                self.EstimateNumBlades = self._EstimateNumBlades(self, "EstimateNumBlades", service, rules, path)
                self.CFDMRowNumList = self._CFDMRowNumList(self, "CFDMRowNumList", service, rules, path)
                self.OldCFDMNumOfBladesList = self._OldCFDMNumOfBladesList(self, "OldCFDMNumOfBladesList", service, rules, path)
                self.NewCFDMNumOfBladesList = self._NewCFDMNumOfBladesList(self, "NewCFDMNumOfBladesList", service, rules, path)
                self.OldCFDMModelBladesList = self._OldCFDMModelBladesList(self, "OldCFDMModelBladesList", service, rules, path)
                self.NewCFDMModelBladesList = self._NewCFDMModelBladesList(self, "NewCFDMModelBladesList", service, rules, path)
                self.OldCFDMAngleOffset = self._OldCFDMAngleOffset(self, "OldCFDMAngleOffset", service, rules, path)
                self.NewCFDMAngleOffset = self._NewCFDMAngleOffset(self, "NewCFDMAngleOffset", service, rules, path)
                self.OldCFDMBladesPerSectorList = self._OldCFDMBladesPerSectorList(self, "OldCFDMBladesPerSectorList", service, rules, path)
                self.NewCFDMBladesPerSectorList = self._NewCFDMBladesPerSectorList(self, "NewCFDMBladesPerSectorList", service, rules, path)

            class _CFDMChildName(PyTextualCommandArgumentsSubItem):
                """
                Argument CFDMChildName.
                """

            class _CFDMSelectMeshAssociation(PyTextualCommandArgumentsSubItem):
                """
                Argument CFDMSelectMeshAssociation.
                """

            class _AxisOfRotation(PyTextualCommandArgumentsSubItem):
                """
                Specify the rotational axis for the generated CFD turbomachine geometry.
                """

            class _DelayCFDModelCreation(PyParameterCommandArgumentsSubItem):
                """
                Argument DelayCFDModelCreation.
                """

            class _RestrictToFactors(PyParameterCommandArgumentsSubItem):
                """
                Choose whether or not to restrict the number of model blade sectors to a factor of the number of blades.
                """

            class _EstimateNumBlades(PyParameterCommandArgumentsSubItem):
                """
                Argument EstimateNumBlades.
                """

            class _CFDMRowNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument CFDMRowNumList.
                """

            class _OldCFDMNumOfBladesList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldCFDMNumOfBladesList.
                """

            class _NewCFDMNumOfBladesList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewCFDMNumOfBladesList.
                """

            class _OldCFDMModelBladesList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldCFDMModelBladesList.
                """

            class _NewCFDMModelBladesList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewCFDMModelBladesList.
                """

            class _OldCFDMAngleOffset(PyTextualCommandArgumentsSubItem):
                """
                Argument OldCFDMAngleOffset.
                """

            class _NewCFDMAngleOffset(PyTextualCommandArgumentsSubItem):
                """
                Argument NewCFDMAngleOffset.
                """

            class _OldCFDMBladesPerSectorList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldCFDMBladesPerSectorList.
                """

            class _NewCFDMBladesPerSectorList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewCFDMBladesPerSectorList.
                """

        def create_instance(self) -> _TWF_CreateCFDModelCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_CreateCFDModelCommandArguments(*args)

    class TWF_ImportMesh(PyCommand):
        """
        Command TWF_ImportMesh.

        Parameters
        ----------
        AddChild : str
        MeshFilePath : str
            Specify the name and location of a single mesh file that includes all the zones, or import multiple mesh files that represent each zone. Standard Ansys mesh file types are supported, including .msh, .msh.h5, .def, .cgns, and .gtm.
        MeshFilePath_old : str
        MeshName : str
        CellZoneNames : list[str]
        ListItemLevels : list[str]
        ListItemTitles : list[str]
        ListOfCellZones : str
        CellZones : list[str]

        Returns
        -------
        bool
        """
        class _TWF_ImportMeshCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.AddChild = self._AddChild(self, "AddChild", service, rules, path)
                self.MeshFilePath = self._MeshFilePath(self, "MeshFilePath", service, rules, path)
                self.MeshFilePath_old = self._MeshFilePath_old(self, "MeshFilePath_old", service, rules, path)
                self.MeshName = self._MeshName(self, "MeshName", service, rules, path)
                self.CellZoneNames = self._CellZoneNames(self, "CellZoneNames", service, rules, path)
                self.ListItemLevels = self._ListItemLevels(self, "ListItemLevels", service, rules, path)
                self.ListItemTitles = self._ListItemTitles(self, "ListItemTitles", service, rules, path)
                self.ListOfCellZones = self._ListOfCellZones(self, "ListOfCellZones", service, rules, path)
                self.CellZones = self._CellZones(self, "CellZones", service, rules, path)

            class _AddChild(PyTextualCommandArgumentsSubItem):
                """
                Argument AddChild.
                """

            class _MeshFilePath(PyTextualCommandArgumentsSubItem):
                """
                Specify the name and location of a single mesh file that includes all the zones, or import multiple mesh files that represent each zone. Standard Ansys mesh file types are supported, including .msh, .msh.h5, .def, .cgns, and .gtm.
                """

            class _MeshFilePath_old(PyTextualCommandArgumentsSubItem):
                """
                Argument MeshFilePath_old.
                """

            class _MeshName(PyTextualCommandArgumentsSubItem):
                """
                Argument MeshName.
                """

            class _CellZoneNames(PyTextualCommandArgumentsSubItem):
                """
                Argument CellZoneNames.
                """

            class _ListItemLevels(PyTextualCommandArgumentsSubItem):
                """
                Argument ListItemLevels.
                """

            class _ListItemTitles(PyTextualCommandArgumentsSubItem):
                """
                Argument ListItemTitles.
                """

            class _ListOfCellZones(PyTextualCommandArgumentsSubItem):
                """
                Argument ListOfCellZones.
                """

            class _CellZones(PyTextualCommandArgumentsSubItem):
                """
                Argument CellZones.
                """

        def create_instance(self) -> _TWF_ImportMeshCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_ImportMeshCommandArguments(*args)

    class TWF_MapRegionInfo(PyCommand):
        """
        Command TWF_MapRegionInfo.

        Parameters
        ----------
        MRChildName : str
        MRSelectCellZone : str
            Select a cell zone for which you wish to review associations.
        UseWireframe : bool
            In order to more easily visualize highlighted items, use this option to display the 3D wireframe representation of the CFD model in the graphics window.
        DefaultMRRegionNameList : list[str]
        DefaultMRFaceZoneList : list[str]
        MRRegionNameList : list[str]
        OldMRFaceZoneList : list[str]
        NewMRFaceZoneList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_MapRegionInfoCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.MRChildName = self._MRChildName(self, "MRChildName", service, rules, path)
                self.MRSelectCellZone = self._MRSelectCellZone(self, "MRSelectCellZone", service, rules, path)
                self.UseWireframe = self._UseWireframe(self, "UseWireframe", service, rules, path)
                self.DefaultMRRegionNameList = self._DefaultMRRegionNameList(self, "DefaultMRRegionNameList", service, rules, path)
                self.DefaultMRFaceZoneList = self._DefaultMRFaceZoneList(self, "DefaultMRFaceZoneList", service, rules, path)
                self.MRRegionNameList = self._MRRegionNameList(self, "MRRegionNameList", service, rules, path)
                self.OldMRFaceZoneList = self._OldMRFaceZoneList(self, "OldMRFaceZoneList", service, rules, path)
                self.NewMRFaceZoneList = self._NewMRFaceZoneList(self, "NewMRFaceZoneList", service, rules, path)

            class _MRChildName(PyTextualCommandArgumentsSubItem):
                """
                Argument MRChildName.
                """

            class _MRSelectCellZone(PyTextualCommandArgumentsSubItem):
                """
                Select a cell zone for which you wish to review associations.
                """

            class _UseWireframe(PyParameterCommandArgumentsSubItem):
                """
                In order to more easily visualize highlighted items, use this option to display the 3D wireframe representation of the CFD model in the graphics window.
                """

            class _DefaultMRRegionNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument DefaultMRRegionNameList.
                """

            class _DefaultMRFaceZoneList(PyTextualCommandArgumentsSubItem):
                """
                Argument DefaultMRFaceZoneList.
                """

            class _MRRegionNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument MRRegionNameList.
                """

            class _OldMRFaceZoneList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldMRFaceZoneList.
                """

            class _NewMRFaceZoneList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewMRFaceZoneList.
                """

        def create_instance(self) -> _TWF_MapRegionInfoCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_MapRegionInfoCommandArguments(*args)

    class TWF_ReportDefMonitors(PyCommand):
        """
        Command TWF_ReportDefMonitors.

        Parameters
        ----------
        RDIsoSurfaceNumList : list[str]
        OldCreateContourList : list[str]
        NewCreateContourList : list[str]
        TurboContoursList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_ReportDefMonitorsCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.RDIsoSurfaceNumList = self._RDIsoSurfaceNumList(self, "RDIsoSurfaceNumList", service, rules, path)
                self.OldCreateContourList = self._OldCreateContourList(self, "OldCreateContourList", service, rules, path)
                self.NewCreateContourList = self._NewCreateContourList(self, "NewCreateContourList", service, rules, path)
                self.TurboContoursList = self._TurboContoursList(self, "TurboContoursList", service, rules, path)

            class _RDIsoSurfaceNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument RDIsoSurfaceNumList.
                """

            class _OldCreateContourList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldCreateContourList.
                """

            class _NewCreateContourList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewCreateContourList.
                """

            class _TurboContoursList(PyTextualCommandArgumentsSubItem):
                """
                Argument TurboContoursList.
                """

        def create_instance(self) -> _TWF_ReportDefMonitorsCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_ReportDefMonitorsCommandArguments(*args)

    class TWF_TurboPhysics(PyCommand):
        """
        Command TWF_TurboPhysics.

        Parameters
        ----------
        States : dict[str, Any]

        Returns
        -------
        bool
        """
        class _TWF_TurboPhysicsCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.States = self._States(self, "States", service, rules, path)

            class _States(PySingletonCommandArgumentsSubItem):
                """
                Argument States.
                """

                def __init__(self, parent, attr, service, rules, path):
                    super().__init__(parent, attr, service, rules, path)
                    self.Density = self._Density(self, "Density", service, rules, path)
                    self.EFM = self._EFM(self, "EFM", service, rules, path)
                    self.Energy = self._Energy(self, "Energy", service, rules, path)
                    self.CEBtn = self._CEBtn(self, "CEBtn", service, rules, path)
                    self.WF = self._WF(self, "WF", service, rules, path)
                    self.OpP = self._OpP(self, "OpP", service, rules, path)
                    self.Vrpm = self._Vrpm(self, "Vrpm", service, rules, path)

                class _Density(PyNumericalCommandArgumentsSubItem):
                    """
                    Provide a value for the density of air, or use the default value.
                    """

                class _EFM(PyTextualCommandArgumentsSubItem):
                    """
                    Displays the current existing fluid assigned to the CFD model. Use the Create/Edit... button to create your own material, or edit other existing materials.
                    """

                class _Energy(PyParameterCommandArgumentsSubItem):
                    """
                    Indicates whether or not temperature conditions are to be considered.
                    """

                class _CEBtn(PyParameterCommandArgumentsSubItem):
                    """
                    Argument CEBtn.
                    """

                class _WF(PyTextualCommandArgumentsSubItem):
                    """
                    Choose one of the following materials as the working fluid for the CFD model.
                    """

                class _OpP(PyNumericalCommandArgumentsSubItem):
                    """
                    Specify the operating pressure, or keep the default value.
                    """

                class _Vrpm(PyNumericalCommandArgumentsSubItem):
                    """
                    Specify the rotation speed, or keep the default value.
                    """

        def create_instance(self) -> _TWF_TurboPhysicsCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_TurboPhysicsCommandArguments(*args)

    class TWF_TurboRegionsZones(PyCommand):
        """
        Command TWF_TurboRegionsZones.

        Parameters
        ----------
        States : dict[str, Any]

        Returns
        -------
        bool
        """
        class _TWF_TurboRegionsZonesCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.States = self._States(self, "States", service, rules, path)

            class _States(PySingletonCommandArgumentsSubItem):
                """
                Argument States.
                """

                def __init__(self, parent, attr, service, rules, path):
                    super().__init__(parent, attr, service, rules, path)
                    self.UseUndo = self._UseUndo(self, "UseUndo", service, rules, path)
                    self.UndoOperationsLog = self._UndoOperationsLog(self, "UndoOperationsLog", service, rules, path)

                class _UseUndo(PyParameterCommandArgumentsSubItem):
                    """
                    Argument UseUndo.
                    """

                class _UndoOperationsLog(PyTextualCommandArgumentsSubItem):
                    """
                    Argument UndoOperationsLog.
                    """

        def create_instance(self) -> _TWF_TurboRegionsZonesCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_TurboRegionsZonesCommandArguments(*args)

    class TWF_TurboSurfaces(PyCommand):
        """
        Command TWF_TurboSurfaces.

        Parameters
        ----------
        NumIsoSurfaces : int
            Specify the number of turbo iso-surfaces you want to create, or keep the default value of 3.
        IsoSurfaceNumList : list[str]
        OldIsoSurfaceNameList : list[str]
        NewIsoSurfaceNameList : list[str]
        OldIsoSurfaceValueList : list[str]
        NewIsoSurfaceValueList : list[str]
        SurfacesList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_TurboSurfacesCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.NumIsoSurfaces = self._NumIsoSurfaces(self, "NumIsoSurfaces", service, rules, path)
                self.IsoSurfaceNumList = self._IsoSurfaceNumList(self, "IsoSurfaceNumList", service, rules, path)
                self.OldIsoSurfaceNameList = self._OldIsoSurfaceNameList(self, "OldIsoSurfaceNameList", service, rules, path)
                self.NewIsoSurfaceNameList = self._NewIsoSurfaceNameList(self, "NewIsoSurfaceNameList", service, rules, path)
                self.OldIsoSurfaceValueList = self._OldIsoSurfaceValueList(self, "OldIsoSurfaceValueList", service, rules, path)
                self.NewIsoSurfaceValueList = self._NewIsoSurfaceValueList(self, "NewIsoSurfaceValueList", service, rules, path)
                self.SurfacesList = self._SurfacesList(self, "SurfacesList", service, rules, path)

            class _NumIsoSurfaces(PyNumericalCommandArgumentsSubItem):
                """
                Specify the number of turbo iso-surfaces you want to create, or keep the default value of 3.
                """

            class _IsoSurfaceNumList(PyTextualCommandArgumentsSubItem):
                """
                Argument IsoSurfaceNumList.
                """

            class _OldIsoSurfaceNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldIsoSurfaceNameList.
                """

            class _NewIsoSurfaceNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewIsoSurfaceNameList.
                """

            class _OldIsoSurfaceValueList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldIsoSurfaceValueList.
                """

            class _NewIsoSurfaceValueList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewIsoSurfaceValueList.
                """

            class _SurfacesList(PyTextualCommandArgumentsSubItem):
                """
                Argument SurfacesList.
                """

        def create_instance(self) -> _TWF_TurboSurfacesCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_TurboSurfacesCommandArguments(*args)

    class TWF_TurboTopology(PyCommand):
        """
        Command TWF_TurboTopology.

        Parameters
        ----------
        TopologyName : str
            Provide a name for the turbo topology, or use the default name.
        UseWireframe : bool
            In order to more easily visualize highlighted items, use this option to display the 3D wireframe representation of the turbo topology model in the graphics window.
        DefaultTopologyNameList : list[str]
        DefaultTopologyZoneList : list[str]
        TopologyNameList : list[str]
        OldTopologyZoneList : list[str]
        NewTopologyZoneList : list[str]

        Returns
        -------
        bool
        """
        class _TWF_TurboTopologyCommandArguments(PyCommandArguments):
            def __init__(self, service, rules, command, path, id):
                super().__init__(service, rules, command, path, id)
                self.TopologyName = self._TopologyName(self, "TopologyName", service, rules, path)
                self.UseWireframe = self._UseWireframe(self, "UseWireframe", service, rules, path)
                self.DefaultTopologyNameList = self._DefaultTopologyNameList(self, "DefaultTopologyNameList", service, rules, path)
                self.DefaultTopologyZoneList = self._DefaultTopologyZoneList(self, "DefaultTopologyZoneList", service, rules, path)
                self.TopologyNameList = self._TopologyNameList(self, "TopologyNameList", service, rules, path)
                self.OldTopologyZoneList = self._OldTopologyZoneList(self, "OldTopologyZoneList", service, rules, path)
                self.NewTopologyZoneList = self._NewTopologyZoneList(self, "NewTopologyZoneList", service, rules, path)

            class _TopologyName(PyTextualCommandArgumentsSubItem):
                """
                Provide a name for the turbo topology, or use the default name.
                """

            class _UseWireframe(PyParameterCommandArgumentsSubItem):
                """
                In order to more easily visualize highlighted items, use this option to display the 3D wireframe representation of the turbo topology model in the graphics window.
                """

            class _DefaultTopologyNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument DefaultTopologyNameList.
                """

            class _DefaultTopologyZoneList(PyTextualCommandArgumentsSubItem):
                """
                Argument DefaultTopologyZoneList.
                """

            class _TopologyNameList(PyTextualCommandArgumentsSubItem):
                """
                Argument TopologyNameList.
                """

            class _OldTopologyZoneList(PyTextualCommandArgumentsSubItem):
                """
                Argument OldTopologyZoneList.
                """

            class _NewTopologyZoneList(PyTextualCommandArgumentsSubItem):
                """
                Argument NewTopologyZoneList.
                """

        def create_instance(self) -> _TWF_TurboTopologyCommandArguments:
            args = self._get_create_instance_args()
            if args is not None:
                return self._TWF_TurboTopologyCommandArguments(*args)




# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a module to search a word through the Fluent's object hierarchy.."""

from collections.abc import Mapping
import fnmatch
import functools
import json
import os
from pathlib import Path
import pickle
import re
import warnings

import ansys.fluent.core as pyfluent
from ansys.fluent.core.solver.error_message import closest_allowed_names
from ansys.fluent.core.utils.fluent_version import (
    FluentVersion,
    get_version_for_file_name,
)


def _get_api_tree_data_file_path():
    """Get API tree data file."""
    from ansys.fluent.core import CODEGEN_OUTDIR

    return (CODEGEN_OUTDIR / "api_tree" / "api_objects.json").resolve()


def get_api_tree_file_name(version: str) -> Path:
    """Get API tree file name."""
    from ansys.fluent.core import CODEGEN_OUTDIR

    return (CODEGEN_OUTDIR / f"api_tree_{version}.pickle").resolve()


def _match(source: str, word: str, match_whole_word: bool, match_case: bool):
    if not match_case:
        source = source.lower()
        word = word.lower()
    if match_whole_word:
        return source == word
    else:
        return word in source


def _remove_suffix(input: str, suffix):
    if hasattr(input, "removesuffix"):
        return input.removesuffix(suffix)
    else:
        if suffix and input.endswith(suffix):
            return input[: -len(suffix)]
        return input


_meshing_rules = ["workflow", "meshing", "PartManagement", "PMFileManagement"]


def _generate_api_data(
    version: str | None = None,
):
    """Generate API tree data.

    Parameters
    ----------
    version : str, optional
        Fluent version to search in. The default is ``None``. If ``None``,
        it searches in the latest version for which codegen was run.
    write_api_tree_data: bool, optional
        Whether to write the API tree data.
    """
    api_objects = set()
    api_tui_objects = set()
    api_object_names = set()
    if version:
        version = get_version_for_file_name(version)
    if not version:
        for fluent_version in FluentVersion:
            version = get_version_for_file_name(fluent_version.value)
            if get_api_tree_file_name(version).exists():
                break
    api_tree_file = get_api_tree_file_name(version)
    with open(api_tree_file, "rb") as f:
        api_tree = pickle.load(f)

    def inner(tree, path):
        for k, v in tree.items():
            if k in ("<meshing_session>", "<solver_session>"):
                next_path = k
            else:
                if k.endswith(":<name>"):
                    k = _remove_suffix(k, ":<name>")
                    next_path = f'{path}.{k}["<name>"]'
                elif k.endswith(":<index>"):
                    k = _remove_suffix(k, ":<index>")
                    next_path = f"{path}.{k}[<index>]"
                else:
                    next_path = f"{path}.{k}"
                type_ = "Object" if isinstance(v, Mapping) else v
                api_object_names.add(k)
                if "tui" in next_path:
                    api_tui_objects.add(f"{next_path} ({type_})")
                else:
                    api_objects.add(f"{next_path} ({type_})")
            if isinstance(v, Mapping):
                inner(v, next_path)

    inner(api_tree, "")

    api_tree_data = dict()
    api_tree_data["api_objects"] = sorted(list(api_objects))
    api_tree_data["api_tui_objects"] = sorted(list(api_tui_objects))
    api_tree_data["all_api_object_names"] = sorted(list(api_object_names))

    def _write_api_tree_file(api_tree_data: dict, api_object_names: list):
        from nltk.corpus import wordnet as wn

        _download_nltk_data()
        from ansys.fluent.core import CODEGEN_OUTDIR

        json_file_folder = Path(os.path.join(CODEGEN_OUTDIR, "api_tree"))
        json_file_folder.mkdir(parents=True, exist_ok=True)

        all_api_object_name_synsets = dict()
        for name in api_object_names:
            api_object_name_synsets = wn.synsets(name, lang="eng")
            synset_names = set()
            for api_object_name_synset in api_object_name_synsets:
                synset_names.add(api_object_name_synset.name())
            if synset_names:
                all_api_object_name_synsets[name] = sorted(list(synset_names))
        api_tree_data["all_api_object_name_synsets"] = all_api_object_name_synsets

        api_tree_file_path = _get_api_tree_data_file_path()
        api_tree_file_path.touch()
        with open(api_tree_file_path, "w") as json_file:
            json.dump(api_tree_data, json_file)

    _write_api_tree_file(
        api_tree_data=api_tree_data, api_object_names=list(api_object_names)
    )
    api_tree_file.unlink()


@functools.cache
def _get_api_tree_data():
    """Get API tree data."""
    api_tree_data_file_path = _get_api_tree_data_file_path()
    if api_tree_data_file_path.exists():
        json_file = open(api_tree_data_file_path, "r")
        api_tree_data = json.load(json_file)
        return api_tree_data


def _print_search_results(queries: list, api_tree_data: dict):
    """Print search results.

    Parameters
    ----------
    queries: list
        List of search string to match API object names.
    api_tree_data: dict
        All API object data.
    """
    results = []
    api_tree_data = api_tree_data if api_tree_data else _get_api_tree_data()
    api_tree_datas = [api_tree_data["api_objects"], api_tree_data["api_tui_objects"]]

    def _get_results(api_tree_data):
        results = []
        for query in queries:
            for api_object in api_tree_data:
                if api_object.split()[0].endswith(query):
                    results.append(api_object)
        return results

    settings_results = _get_results(api_tree_datas[0])
    tui_results = _get_results(api_tree_datas[1])

    settings_results.sort()
    tui_results.sort()

    results.extend(settings_results)
    results.extend(tui_results)

    if pyfluent.PRINT_SEARCH_RESULTS:
        for result in results:
            print(result)
    elif results:
        return results


def _get_wildcard_matches_for_word_from_names(word: str, names: list):
    """Get wildcard matches for the given word.

    Parameters
    ----------
    word: str
       Word to search for.
    names: list
        All API object names.

    Returns
    -------
    wildcard_matches: list
        Matched API object names.
    """
    pattern = fnmatch.translate(word)
    regex = re.compile(pattern)
    return [name for name in names if regex.match(name)]


def _search_wildcard(search_string: str, api_tree_data: dict):
    """Perform wildcard search for a word through the Fluent's object hierarchy.

    Parameters
    ----------
    search_string: str
        Word to search for. Semantic search is default.
    api_tree_data: dict
        All API object data.

    Returns
    -------
        List of search string matches.
    """
    api_tree_data = api_tree_data if api_tree_data else _get_api_tree_data()
    queries = _get_wildcard_matches_for_word_from_names(
        search_string, names=api_tree_data["all_api_object_names"]
    )
    if queries:
        return _print_search_results(queries, api_tree_data=api_tree_data)


def _get_exact_match_for_word_from_names(
    word: str,
    names: list,
):
    """Get exact match for the given word.

    Parameters
    ----------
    word: str
        Word to search for.
    names: list
        All API object names.

    Returns
    -------
        List of exact match.
    """
    return list({name for name in names if word == name or word in name})


def _get_capitalize_match_for_word_from_names(
    word: str,
    names: list,
):
    """Get API object name if capitalize case of the given word is present in the API
    object name.

    Parameters
    ----------
    word: str
        Word to search for.
    names: list
        All API object names.

    Returns
    -------
        List of API object names containing capitalize case of the given word.
    """
    return [name for name in names if word.capitalize() in name]


def _get_match_case_for_word_from_names(
    word: str,
    names: list,
):
    """Get API object name if the given word is present in the API object name.

    Parameters
    ----------
    word: str
        Word to search for.
    names: list
        All API object names.

    Returns
    -------
        List of API object names containing the given word.
    """
    return [name for name in names if word in name]


def _get_close_matches_for_word_from_names(
    word: str,
    names: list,
):
    """Get close matches for the given word.

    Parameters
    ----------
    word: str
        Word to search for.
    names: list
        All API object names.

    Returns
    -------
    valid_close_matches: list
        List of valid close matches.
    """
    close_matches = closest_allowed_names(word, names)
    valid_close_matches = [
        close_match for close_match in close_matches if close_match in names
    ]
    return valid_close_matches


def _search_whole_word(
    search_string: str,
    match_case: bool = False,
    match_whole_word: bool = True,
    api_tree_data: dict = None,
):
    """Perform exact search for a word through the Fluent's object hierarchy.

    Parameters
    ----------
    search_string: str
        Word to search for. Semantic search is default.
    match_case: bool
        Whether to match case. The default is ``False``.
        If ``True``, it matches the given word.
    match_whole_word: bool
        Whether to match whole word. The default is ``False``.
        If ``True``, it matches the given word, and it's capitalize case.
    api_tree_data: dict
        All API object data.

    Returns
    -------
        List of search string matches.
    """
    api_tree_data = api_tree_data if api_tree_data else _get_api_tree_data()
    queries = []
    if not match_case and not match_whole_word:
        queries.extend(
            _get_capitalize_match_for_word_from_names(
                search_string,
                names=api_tree_data["all_api_object_names"],
            )
        )
        queries.extend(
            _get_match_case_for_word_from_names(
                search_string,
                names=api_tree_data["all_api_object_names"],
            )
        )
    elif match_case and match_whole_word:
        queries.extend(
            _get_exact_match_for_word_from_names(
                search_string,
                names=api_tree_data["all_api_object_names"],
            )
        )
    elif match_case:
        queries.extend(
            _get_match_case_for_word_from_names(
                search_string,
                names=api_tree_data["all_api_object_names"],
            )
        )
    elif match_whole_word:
        for word in [search_string, search_string.capitalize()]:
            queries.extend(
                _get_exact_match_for_word_from_names(
                    word,
                    names=api_tree_data["all_api_object_names"],
                )
            )
    if queries:
        return _print_search_results(queries, api_tree_data=api_tree_data)


def _download_nltk_data():
    """Download NLTK data on demand."""
    import ssl

    import nltk

    try:
        _create_unverified_context = ssl._create_unverified_context
    except AttributeError:
        pass
    else:
        ssl._create_default_https_context = _create_unverified_context

    packages = ["wordnet", "omw-1.4"]
    for package in packages:
        nltk.download(
            package,
            quiet=True,
            halt_on_error=False,
        )


def _search_semantic(search_string: str, language: str, api_tree_data: dict):
    """Perform semantic search for a word through the Fluent's object hierarchy.

    Parameters
    ----------
    search_string: str
        Word to search for. Semantic search is the default.
    language: str
        ISO 639-3 code for the language to use for the semantic search.
        The default is ``eng`` for English. For the list of supported languages,
        see `OMW Version 1 <https://omwn.org/omw1.html>`_.
    api_tree_data: dict
        All API object data.

    Returns
    -------
    queries: list
        List of search string matches.
    """
    from nltk.corpus import wordnet as wn

    api_tree_data = api_tree_data if api_tree_data else _get_api_tree_data()
    similar_keys = set()
    search_string_synsets = set(wn.synsets(search_string, lang=language))
    for api_object_name, api_object_synset_names in list(
        api_tree_data["all_api_object_name_synsets"].items()
    ):
        api_object_synsets = {
            wn.synset(api_object_synset_name)
            for api_object_synset_name in api_object_synset_names
        }
        if search_string_synsets & api_object_synsets:
            similar_keys.add(api_object_name + "*")
    if similar_keys:
        results = []
        for key in similar_keys:
            result = _search_wildcard(key, api_tree_data)
            if result:
                results.extend(result)
        if results:
            return results
    else:
        queries = _get_close_matches_for_word_from_names(
            search_string,
            names=api_tree_data["all_api_object_names"],
        )
        if queries:
            return _print_search_results(queries, api_tree_data=api_tree_data)


def search(
    search_string: str,
    language: str | None = "eng",
    wildcard: bool | None = False,
    match_whole_word: bool = False,
    match_case: bool | None = True,
):
    """Search for a word through the Fluent's object hierarchy.

    Parameters
    ----------
    search_string: str
        Word to search for. Semantic search is the default.
    language: str
        ISO 639-3 code for the language to use for the semantic search.
        The default is ``eng`` for English. For the list of supported languages,
        see `OMW Version 1 <https://omwn.org/omw1.html>`_.
    wildcard: bool, optional
        Whether to use the wildcard pattern. The default is ``False``. If ``True``, the
        wildcard pattern is based on the ``fnmatch`` module and semantic matching
        is turned off.
    match_whole_word: bool, optional
        Whether to find only exact matches. The default is ``False``. If ``True``,
        only exact matches are found and semantic matching is turned off.
    match_case: bool, optional
        Whether to match case. The default is ``True``. If ``False``, the search is case-insensitive.

    Examples
    --------
    >>> import ansys.fluent.core as pyfluent
    >>> pyfluent.search("font", match_whole_word=True)
    >>> pyfluent.search("Font")
    >>> pyfluent.search("iter*", wildcard=True)
    >>> pyfluent.search("读", language="cmn")   # search 'read' in Chinese
    The most similar API objects are:
    <solver_session>.file.read (Command)
    <solver_session>.file.import_.read (Command)
    <solver_session>.mesh.surface_mesh.read (Command)
    <solver_session>.tui.display.display_states.read (Command)
    <meshing_session>.tui.display.display_states.read (Command)
    """
    if (wildcard and match_whole_word) or (wildcard and match_case):
        warnings.warn(
            "``wildcard=True`` matches wildcard pattern.",
            UserWarning,
        )
    elif language and wildcard:
        warnings.warn(
            "``wildcard=True`` matches wildcard pattern.",
            UserWarning,
        )

    api_tree_data = _get_api_tree_data()

    if wildcard:
        return _search_wildcard(
            search_string,
            api_tree_data=api_tree_data,
        )
    elif match_whole_word:
        if not match_case:
            return _search_whole_word(
                search_string, match_whole_word=True, api_tree_data=api_tree_data
            )
        else:
            return _search_whole_word(
                search_string,
                match_case=True,
                match_whole_word=True,
                api_tree_data=api_tree_data,
            )
    else:
        try:
            return _search_semantic(
                search_string, language, api_tree_data=api_tree_data
            )
        except ModuleNotFoundError:
            pass
        except LookupError:
            _download_nltk_data()
            return _search_semantic(
                search_string, language, api_tree_data=api_tree_data
            )



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Create certificates for gRPC mutual TLS testing, including support for HPC deployments.

This script generates a complete set of certificates including:
- Certificate Authority (CA) key and certificate
- Server key and certificate(s) (signed by CA)
- Client key and certificate (signed by CA)

All certificates are generated with appropriate extensions for their intended use.

Usage:
    python generate_certs.py [options]
    python -m generate_certs [options]

Examples:
    # Simple usage (creates localhost server cert)
    python generate_certs.py

    # Single custom server
    python generate_certs.py --server myserver

    # Custom client name
    python generate_certs.py --server localhost --client "My Client"

    # Custom certificate validity period (1 year)
    python generate_certs.py --server localhost --days 365

    # HPC deployment with multiple servers
    python -m generate_certs --server node01,192.0.2.1 --server node02,192.0.2.2 --server node23,192.0.2.23

    # Mixed server configurations with custom validity
    python generate_certs.py --server main-server --server worker01 --server worker02,10.0.1.2 --days 1825
"""

import argparse
from datetime import datetime, timedelta, timezone
import os
from pathlib import Path
import sys

from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509.oid import ExtendedKeyUsageOID, NameOID


def generate_private_key(key_size: int = 4096) -> rsa.RSAPrivateKey:
    """
    Generate an RSA private key.

    Parameters
    ----------
    key_size : int, optional
        Size of the RSA key in bits, by default 4096

    Returns
    -------
    rsa.RSAPrivateKey
        Generated RSA private key
    """
    return rsa.generate_private_key(
        public_exponent=65537,
        key_size=key_size,
    )


def save_private_key(key: rsa.RSAPrivateKey, filename: str) -> None:
    """
    Save a private key to a PEM file.

    Parameters
    ----------
    key : rsa.RSAPrivateKey
        The private key to save
    filename : str
        Path to the output file
    """
    with open(filename, "wb") as f:
        f.write(
            key.private_bytes(
                encoding=serialization.Encoding.PEM,
                format=serialization.PrivateFormat.PKCS8,
                encryption_algorithm=serialization.NoEncryption(),
            )
        )


def save_certificate(cert: x509.Certificate, filename: str) -> None:
    """
    Save a certificate to a PEM file.

    Parameters
    ----------
    cert : x509.Certificate
        The certificate to save
    filename : str
        Path to the output file
    """
    with open(filename, "wb") as f:
        f.write(cert.public_bytes(serialization.Encoding.PEM))


def create_ca_certificate(
    ca_key: rsa.RSAPrivateKey, validity_days: int
) -> x509.Certificate:
    """
    Create a self-signed CA certificate.

    Parameters
    ----------
    ca_key : rsa.RSAPrivateKey
        The private key for the CA certificate
    validity_days : int
        Number of days the certificate should be valid

    Returns
    -------
    x509.Certificate
        Self-signed CA certificate with appropriate extensions for certificate signing
    """
    subject = issuer = x509.Name(
        [
            x509.NameAttribute(NameOID.COMMON_NAME, "Test CA"),
        ]
    )

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(ca_key.public_key())
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.now(timezone.utc))
        .not_valid_after(datetime.now(timezone.utc) + timedelta(days=validity_days))
        .add_extension(
            x509.BasicConstraints(ca=True, path_length=None),
            critical=True,
        )
        .add_extension(
            x509.KeyUsage(
                digital_signature=True,
                key_cert_sign=True,
                crl_sign=True,
                key_encipherment=False,
                data_encipherment=False,
                key_agreement=False,
                content_commitment=False,
                encipher_only=False,
                decipher_only=False,
            ),
            critical=True,
        )
        .sign(ca_key, hashes.SHA256())
    )

    return cert


def create_server_certificate(
    server_key: rsa.RSAPrivateKey,
    ca_cert: x509.Certificate,
    ca_key: rsa.RSAPrivateKey,
    server_common_name: str,
    validity_days: int,
    san_names: list = None,
) -> x509.Certificate:
    """
    Create a server certificate signed by the CA with optional Subject Alternative Names.

    Parameters
    ----------
    server_key : rsa.RSAPrivateKey
        The private key for the server certificate
    ca_cert : x509.Certificate
        The CA certificate to use as issuer
    ca_key : rsa.RSAPrivateKey
        The CA private key to sign the certificate
    server_common_name : str
        The common name for the server certificate (will be used as CN and primary SAN)
    validity_days : int
        Number of days the certificate should be valid
    san_names : list, optional
        Additional Subject Alternative Names to include, by default None

    Returns
    -------
    x509.Certificate
        Server certificate signed by the CA with SERVER_AUTH extended key usage
    """
    subject = x509.Name(
        [
            x509.NameAttribute(NameOID.COMMON_NAME, server_common_name),
        ]
    )

    # Build SAN list - always include the CN, plus any additional names
    san_list = [x509.DNSName(server_common_name)]
    if san_names:
        for name in san_names:
            # Skip if it's the same as CN to avoid duplicates
            if name != server_common_name:
                san_list.append(x509.DNSName(name))

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(ca_cert.subject)
        .public_key(server_key.public_key())
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.now(timezone.utc))
        .not_valid_after(datetime.now(timezone.utc) + timedelta(days=validity_days))
        .add_extension(
            x509.SubjectAlternativeName(san_list),
            critical=False,
        )
        .add_extension(
            x509.KeyUsage(
                digital_signature=True,
                key_encipherment=True,
                key_cert_sign=False,
                crl_sign=False,
                data_encipherment=False,
                key_agreement=False,
                content_commitment=False,
                encipher_only=False,
                decipher_only=False,
            ),
            critical=True,
        )
        .add_extension(
            x509.ExtendedKeyUsage(
                [
                    ExtendedKeyUsageOID.SERVER_AUTH,
                ]
            ),
            critical=False,
        )
        .sign(ca_key, hashes.SHA256())
    )

    return cert


def create_client_certificate(
    client_key: rsa.RSAPrivateKey,
    ca_cert: x509.Certificate,
    ca_key: rsa.RSAPrivateKey,
    client_common_name: str,
    validity_days: int,
) -> x509.Certificate:
    """
    Create a client certificate signed by the CA.

    Parameters
    ----------
    client_key : rsa.RSAPrivateKey
        The private key for the client certificate
    ca_cert : x509.Certificate
        The CA certificate to use as issuer
    ca_key : rsa.RSAPrivateKey
        The CA private key to sign the certificate
    client_common_name : str
        The common name for the client certificate
    validity_days : int
        Number of days the certificate should be valid

    Returns
    -------
    x509.Certificate
        Client certificate signed by the CA with CLIENT_AUTH extended key usage
    """
    subject = x509.Name(
        [
            x509.NameAttribute(NameOID.COMMON_NAME, client_common_name),
        ]
    )

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(ca_cert.subject)
        .public_key(client_key.public_key())
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.now(timezone.utc))
        .not_valid_after(datetime.now(timezone.utc) + timedelta(days=validity_days))
        .add_extension(
            x509.SubjectAlternativeName(
                [
                    x509.DNSName(client_common_name),
                ]
            ),
            critical=False,
        )
        .add_extension(
            x509.KeyUsage(
                digital_signature=True,
                key_encipherment=True,
                key_cert_sign=False,
                crl_sign=False,
                data_encipherment=False,
                key_agreement=False,
                content_commitment=False,
                encipher_only=False,
                decipher_only=False,
            ),
            critical=True,
        )
        .add_extension(
            x509.ExtendedKeyUsage(
                [
                    ExtendedKeyUsageOID.CLIENT_AUTH,
                ]
            ),
            critical=False,
        )
        .sign(ca_key, hashes.SHA256())
    )

    return cert


def parse_server_spec(server_spec: str) -> tuple[str, list]:
    """
    Parse a server specification string into primary hostname and SAN list.

    Parameters
    ----------
    server_spec : str
        A comma-separated string like "node01,192.0.2.1" or just "node01"

    Returns
    -------
    tuple[str, list]
        Tuple containing (primary_hostname, [additional_san_names])

    Raises
    ------
    ValueError
        If the server specification is empty or invalid
    """
    names = [name.strip() for name in server_spec.split(",") if name.strip()]
    if not names:
        raise ValueError("Server specification cannot be empty")

    primary_hostname = names[0]
    additional_sans = names[1:] if len(names) > 1 else []

    return primary_hostname, additional_sans


def generate_server_certificates(
    ca_cert: x509.Certificate,
    ca_key: rsa.RSAPrivateKey,
    server_specs: list,
    validity_days: int,
) -> list:
    """
    Generate multiple server certificates based on server specifications.

    Parameters
    ----------
    ca_cert : x509.Certificate
        The CA certificate to sign with
    ca_key : rsa.RSAPrivateKey
        The CA private key to sign with
    server_specs : list
        List of server specification strings in format "hostname[,san1,san2,...]"
    validity_days : int
        Number of days the certificates should be valid

    Returns
    -------
    list
        List of generated certificate filenames
    """
    generated_files = []

    for spec in server_specs:
        primary_hostname, additional_sans = parse_server_spec(spec)

        # If only one server is specified, use 'server' as generic name
        if len(server_specs) == 1:
            filename = "server"
        else:
            filename = primary_hostname

        print(f"Generating server certificate for {primary_hostname}")
        if additional_sans:
            print(f"  Additional SAN names: {', '.join(additional_sans)}")

        # Generate server key and certificate
        server_key = generate_private_key()
        server_cert = create_server_certificate(
            server_key,
            ca_cert,
            ca_key,
            primary_hostname,
            validity_days,
            additional_sans,
        )

        # Save with primary hostname as filename
        key_filename = f"{filename}.key"
        cert_filename = f"{filename}.crt"

        save_private_key(server_key, key_filename)
        save_certificate(server_cert, cert_filename)

        generated_files.extend([key_filename, cert_filename])

    return generated_files


def main():
    """
    Generate all certificates based on command-line arguments.

    This function serves as the main entry point for the certificate generation tool.
    It parses command-line arguments and generates a complete PKI setup including
    CA, server, and client certificates suitable for gRPC mutual TLS.
    """
    # Parse command-line arguments
    parser = argparse.ArgumentParser(
        description="Generate certificates for gRPC mutual TLS testing",
        epilog="""
Examples:
  %(prog)s --server localhost
  %(prog)s --server node01,192.0.2.1 --server node02,192.0.2.2 --server node23,192.0.2.23
  %(prog)s --server main-server --server worker01 --server worker02,10.0.1.2
  %(prog)s --server myserver --client "My Client"
  %(prog)s --server localhost --days 365
        """,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "--server",
        action="append",
        dest="servers",
        help="Server specification: hostname[,san1,san2,...] (can be used multiple times) (default: localhost)",
    )
    parser.add_argument(
        "--client",
        default="Test Client",
        help="Common name for the client certificate (default: Test Client)",
    )
    parser.add_argument(
        "--days",
        type=int,
        default=3650,
        help="Number of days the certificates should be valid (default: 3650, which is 10 years)",
    )
    parser.add_argument(
        "--output-dir",
        type=Path,
        default=Path("certs"),
        help="Output directory for certificates (default: certs)",
    )

    args = parser.parse_args()

    # Validate arguments
    if not args.servers:
        # Default behavior: create localhost server cert
        args.servers = ["localhost,127.0.0.1"]

    # Create output directory if it doesn't exist
    output_dir = args.output_dir
    output_dir.mkdir(exist_ok=True)

    # Change to output directory
    original_dir = os.getcwd()
    os.chdir(output_dir)

    try:
        generated_files = []

        print("Generating CA key and certificate for self-signing...")
        # Generate CA key and certificate for self-signing
        ca_key = generate_private_key()
        ca_cert = create_ca_certificate(ca_key, args.days)

        save_private_key(ca_key, "ca.key")
        save_certificate(ca_cert, "ca.crt")
        generated_files.extend(["ca.key", "ca.crt"])

        # Generate server certificates
        print(f"Generating {len(args.servers)} server certificate(s)...")
        server_files = generate_server_certificates(
            ca_cert, ca_key, args.servers, args.days
        )
        generated_files.extend(server_files)

        print(f"Generating client certificate (CN: {args.client})...")
        # Generate client key and certificate
        client_key = generate_private_key()
        client_cert = create_client_certificate(
            client_key, ca_cert, ca_key, args.client, args.days
        )

        save_private_key(client_key, "client.key")
        save_certificate(client_cert, "client.crt")
        generated_files.extend(["client.key", "client.crt"])

        print(f"\nDone! Generated {len(generated_files)} files in {output_dir}:")
        for filename in sorted(generated_files):
            print(f"  {filename}")

    except Exception as e:
        print(f"Error generating certificates: {e}", file=sys.stderr)
        sys.exit(1)
    finally:
        # Return to original directory
        os.chdir(original_dir)


if __name__ == "__main__":
    main()



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Utility functions for working with Fluent Docker images.
"""


import os


def _is_grpc_patched(image_tag: str):
    """
    Check if the image tag corresponds to a version that patches gRPC.
    """
    min_patched_versions = {
        "v25.2": 3,
        "v25.1": 4,
        "v24.2": 5,
    }
    for version, min_patch in min_patched_versions.items():
        if image_tag.startswith(version) and image_tag.count(".") == 2:
            patch_version = int(image_tag.split(".")[-1])
            return patch_version >= min_patch
    return False


def get_grpc_launcher_args_for_gh_runs():
    """
    Get the gRPC launcher arguments for GitHub Actions runs based on the Fluent image tag.
    """
    kwds = {}
    fluent_image_tag = os.getenv("FLUENT_IMAGE_TAG")
    if fluent_image_tag:
        if _is_grpc_patched(fluent_image_tag) and not fluent_image_tag.startswith(
            "v24.2"
        ):
            kwds["certificates_folder"] = os.path.join(os.getcwd(), "certs")
        else:
            kwds["insecure_mode"] = True
    return kwds


def get_ghcr_fluent_image_name(image_tag: str):
    """
    Get the Fluent image name from GitHub registry based on the image tag.
    """
    if (
        image_tag.startswith("sha256:")
        or image_tag >= "v26.1"
        or _is_grpc_patched(image_tag)
    ):
        return "ghcr.io/ansys/fluent"
    else:
        return "ghcr.io/ansys/pyfluent"



# Copyright (C) 2021 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import os
from pathlib import Path

import pytest

import ansys.fluent.core as pyfluent


@pytest.mark.settings_only
@pytest.mark.fluent_version("latest")
def test_solver_import_mixingelbow(mixing_elbow_settings_session):
    solver_session = mixing_elbow_settings_session
    assert solver_session.settings.is_active()
    assert solver_session.is_server_healthy()
    file_name = Path(pyfluent.EXAMPLES_PATH) / "jou_test_general.py"
    solver_session.journal.start(file_name.as_posix())
    ###
    assert solver_session.setup.models.energy.enabled()
    scheme_eval = solver_session.scheme.eval
    assert scheme_eval("(case-valid?)")
    ###
    solver_session.tui.mesh.check()
    solver_session.tui.define.units("length", "in")
    assert scheme_eval('(units/quantity-info "length")')[-1] == "in"
    general_solver = solver_session.setup.general.solver
    general_solver.time.allowed_values()
    assert general_solver.time.allowed_values() == [
        "steady",
        "unsteady-1st-order",
        "unsteady-2nd-order",
        "unsteady-2nd-order-bounded",
    ]

    general_solver.time = "unsteady-2nd-order"
    general_solver.time = "unsteady-1st-order"
    general_solver.time = "unsteady-2nd-order-bounded"
    general_solver.time = "steady"

    # solver.setup.general.gravity = {"gravity": True, "y_component": -9.81}
    # solver.mesh.scale(x_scale=0.001, y_scale=0.001, z_scale=0.001)

    assert general_solver.type.get_attr("allowed-values") == [
        "pressure-based",
        "density-based-implicit",
        "density-based-explicit",
    ]
    assert general_solver.type.allowed_values() == [
        "pressure-based",
        "density-based-implicit",
        "density-based-explicit",
    ]
    # Below line is commented due to TFS Bug 714494
    # assert solver_session.setup.general.solver.type.default_value() == "pressure-based"
    assert general_solver.type.is_active()
    assert not general_solver.type.is_read_only()
    general_solver.type = "density-based-implicit"
    assert general_solver.type() == "density-based-implicit"
    general_solver.type = "density-based-explicit"
    assert general_solver.type() == "density-based-explicit"
    general_solver.type = "pressure-based"
    assert general_solver.type() == "pressure-based"

    auto_save = solver_session.file.auto_save
    auto_save.data_frequency = 10
    assert auto_save.data_frequency.default_value() == 0
    assert auto_save.data_frequency() == 10
    auto_save.case_frequency = "each-time"
    assert auto_save.case_frequency() == "each-time"
    auto_save.root_name = "file_auto_save"
    assert auto_save.root_name() == "file_auto_save"
    solver_session.setup.reference_values.compute(
        from_zone_name="outlet", from_zone_type="pressure-outlet"
    )
    solver_session.journal.stop()
    solver_session.tui.file.read_journal(file_name.as_posix())
    assert auto_save.root_name() == "file_auto_save"
    assert general_solver.type() == "pressure-based"
    assert auto_save.data_frequency() == 10
    assert general_solver.time() == "steady"
    if os.path.exists(file_name):
        os.remove(file_name)


@pytest.mark.settings_only
@pytest.mark.fluent_version("latest")
def test_disk_2d_setup(disk_settings_session):
    session = disk_settings_session
    assert session.settings.is_active()
    assert session.is_server_healthy()
    ###
    assert not session.setup.models.energy.enabled()
    assert session.scheme.eval("(case-valid?)")
    session.tui.mesh.check()

    session_solver = session.setup.general.solver
    assert session_solver.two_dim_space.get_attr("allowed-values") == [
        "swirl",
        "axisymmetric",
        "planar",
    ]
    assert session_solver.two_dim_space() == "planar"
    session_solver.two_dim_space = "axisymmetric"
    assert session_solver.two_dim_space() == "axisymmetric"
    session_solver.two_dim_space = "swirl"
    assert session_solver.two_dim_space() == "swirl"
    session_solver.two_dim_space = "planar"
    assert session_solver.two_dim_space() == "planar"
    # Bug 682773
    # session.setup.general.gravity = {"gravity": True, "x_component": -9.81}
